#!/bin/bash
########################################################
###
### Draw resolution matrix figure
###
###         by Meijian An, meijianan@live.com
###               Jan, 2012
###
########################################################

#### ---------------------------
#### defaults
#### ---------------------------
resf=   #Rmatrix.dat
res=R_matrix
G=
abs=no
nobottom=no
shade="polar -I"   ###same parameter as for -C in makecpt of GMT

inform="-resf=A_matrix -G=G_matrix -abs -nobottom"
for (( i=1; i<=$#; i++ )); do
	argu="${!i}"
	case "${argu%%=*}" in
	"-resf")         resf="${argu#*=}";;
	"-G")               G="${argu#*=}";;
	"-abs")           abs=yes;;
	"-nobottom") nobottom=yes;;
	"-shade")       shade="${argu#*=}";;
	esac
done

if [ "$resf" != "" ]; then
	res=${resf%.dat}
fi

ypos=9
if [ "$G" != "" ] && [ "$G" != "-" ]; then
	ypos=2
fi

rm -f .gmt*
gmtset MEASURE_UNIT cm
gmtset CHAR_ENCODING Standard+
gmtset BASEMAP_TYPE FANCY
gmtset PLOT_DEGREE_FORMAT ddd:mm:ssF
gmtset FRAME_PEN 1p FRAME_WIDTH 0.1c
gmtset HEADER_FONT_SIZE	 14p	HEADER_OFFSET 0.6c  HEADER_FONT Times-Bold
gmtset TICK_LENGTH 0.15c TICK_PEN 1p
gmtset ANOT_FONT_SIZE 14p ANOT_OFFSET 0.1c 
gmtset LABEL_FONT_SIZE 14p LABEL_OFFSET 0c
gmtset COLOR_NAN 225/225/225 COLOR_FOREGROUND 200/200/200
#gmtset GRID_CROSS_SIZE 0
gmtset GRID_PEN  0.1p,gray

###------------------------------------------------------
###            Draw normalized resolution matrix
###------------------------------------------------------
output=${res}_Normalized.ps

makecpt -C$shade -T-1/1/0.05 >res.cpt

rm -f Rmatrix.tmp
awk '{
	irow=NR
	maxv=-100
	ncol = ("'${nobottom}'"=="no")?NF:(NF-1)
	
	for(icol=1; icol<=ncol; icol++)
	{
			
		if("'${abs}'" == "yes")	
			absv=($icol > 0)?$icol:(-$icol)
		else
			absv=$icol
		if(maxv < absv) maxv=absv
	}
	if(maxv == 0) maxv=1.0
	
	if( $irow < 0 && "'${abs}'" == "yes" )
		maxv=-maxv
	for(icol=1; icol<=ncol; icol++)
	{
		print icol, irow, $(icol)/maxv # ith x is y axis
		
		print icol, irow, $(icol) > "Rmatrix.tmp"
	}
}END{
	print ncol, irow >"param.tmp"
}'  ${resf} >Rmatrix.tmp_normal

mparam=`awk '{print $1}' param.tmp`
rm -f param.tmp

xyz2grd Rmatrix.tmp_normal -R1/${mparam}/1/${mparam} -I1 -G${res}.grd
labels=(`echo $mparam | awk '{str=sprintf("%e", $1); split(str, aa, "e"); v=0.5; if(aa[1]>=6) v=2; else if(aa[1]>3)v=1; fstr=v "e" aa[2]; fv=fstr*1.0; printf "a%df%dg%d", fv, fv/10, fv/2}'`)
range=`echo ${mparam} | awk '{printf "0.5/%lf/0.5/%lf", $1+0.5, $1+0.5}'`
#echo ${labels[*]}
#labels=( a20f1g10 )

grdimage ${res}.grd -Cres.cpt -R$range -JX15/-15 -BNsWe${labels[0]}:"Parameter index (@%6%j@%%)":/${labels[0]}:"Parameter index (@%6%i@%%)"::."Normalized resolution matrix":  -X3 -Y${ypos} -K -P > ${output}
grdcontour ${res}.grd -R -J -C0.5 -A0.5+s14 -L0.45/0.55 -W6,black,- -O -K >> ${output}

if [ -f resolution_length.info ]; then
	awk '{
		for(i=$1; i<=$2; i+=1)
		{ 
			print ">"; 
			print i+$3, i; 
			print i-$3, i; 
			print ">"; 
			print i-$3, i-0.5; 
			print i-$3, i+0.5; 
			print ">";  
			print i+$3, i-0.5; 
			print i+$3, i+0.5 
		} 
	}' resolution_length.info |\
		psxy -R -J -W3,red,- -m -O -K >>${output}
fi

cat <<eof | psxy -R -J -W1,0,- -O -K >> ${output}
0 0
$[mparam+1] $[mparam+1]
eof
psscale -Cres.cpt -D3.3/0/7.5/0.3h -Ba1f0.5 -X4 -Y-0.6 -O -K >> ${output}

if [ "$G" != "" ] && [ "$G" != "-" ]; then
	nob=`awk 'END{print NR}' $G`
	awk '{for(i=1; i<=NF; i++) print i, NR, $i}' $G |\
		xyz2grd -R1/${mparam}/1/$nob -I1 -G${G}_tmp.grd
	
	ymax=(`grdinfo ${G}_tmp.grd | awk '($2=="z_min:"){mi=($3>0)?$3:(-$3); mx=($5>0)?$5:(-$5); out=(mx>mi)?mx:mi; print out, out/40}'`)
	makecpt -Cpolar -T-${ymax[0]}/${ymax[0]}/${ymax[1]} >${G}_tmp.cpt
	
	hgt=`echo "$nob / 5" | bc`
	if [ $nob -gt 30 ]; then
		hgt=6
	fi
	obmax=`echo "$nob + 0.5" | bc`
	grdimage ${G}_tmp.grd -C${G}_tmp.cpt -R1/${mparam}/0.5/$obmax -JX15/${hgt} -BNsWe${labels[0]}:"Parameter index":/a5f1:"Observation index"::."Observation matrix G":  -X-4 -Y19.5 -O -K>> ${output}
	psscale -C${G}_tmp.cpt -D3.3/0/7.5/0.3h -Ba1 -X4 -Y-0.6 -O >> ${output}
fi

ps2pdf ${output}
xpdf ${output%.ps}.pdf&
rm -f ${output} ${res}.grd res.cpt

###------------------------------------------------------
### Draw resolution matrix
###------------------------------------------------------
resf=Rmatrix.tmp
output=${res}.ps

minmax=(`awk '{print $3}' ${resf} | minmax -C | awk '{d=$2/10; if($2>=0.9) print -1.1, 1.1, 0.05; else printf "%.2lf %.2lf %.3lf\n", -($2+d), $2+d, ($2+d)/20.0}'`)
makecpt -C$shade -T${minmax[0]}/${minmax[1]}/${minmax[2]} >res.cpt
xyz2grd Rmatrix.tmp -R1/${mparam}/1/${mparam} -I1 -G${res}.grd

range=`echo ${mparam} | awk '{printf "0.5/%lf/0.5/%lf", $1+0.5, $1+0.5}'`
grdimage ${res}.grd -Cres.cpt -R$range -JX15/-15 -BNsWe${labels[0]}:"Parameter index (@%6%j@%%)":/${labels[0]}:"Parameter index (@%6%i@%%)"::."Resolution matrix":  -X3 -Y${ypos} -K -P > ${output}
cat <<eof | psxy -R -J -W1,0,- -O -K >> ${output}
0 0
$[mparam+1] $[mparam+1]
eof
psscale -Cres.cpt -D3.3/0/7.5/0.3h -Ba${minmax[1]} -X4 -Y-0.6 -O -K >> ${output}

if [ "$G" != "" ] && [ "$G" != "-" ]; then
	grdimage ${G}_tmp.grd -C${G}_tmp.cpt -R1/${mparam}/0.5/$obmax -JX15/${hgt} -BNsWe${labels[0]}:"Parameter index":/a5f1:"Observation index"::."Observation matrix G":  -X-4 -Y19.5 -O -K>> ${output}
	psscale -C${G}_tmp.cpt -D3.3/0/7.5/0.3h -Ba1f0.5 -X4 -Y-0.6 -O >> ${output}
fi

ps2pdf ${output}
xpdf ${output%.ps}.pdf&
rm -f ${output} ${res}.grd res.cpt Rmatrix.tmp_normal Rmatrix.tmp .gmt*
rm -f ${G}_tmp.grd ${G}_tmp.cpt
