#!/bin/bash
############################################################
###
### obtain hybrid resolution matrix by SVD
###           original equation:    G * X = D
###           regularized equation: A * X = B
###           hybrid resolution matrix: R = A^-1 * G
###   argus:
###           -A=?  A matrix file
###           -G=?  G vector file
###
###         by Meijian An, meijianan@live.com
###               Jan, 2012
############################################################

A=-
G=-

inform="-A=A_matrix -G=G_matrix (G*X=D and A*X=B)"
for (( i=1; i<=$#; i++ )); do
	argu="${!i}"
	case "${argu%%=*}" in
	"-A")      A="${argu#*=}";;
	"-G")      G="${argu#*=}";;
	esac
done

if [ ! -f "${A}" ] || [ ! -f "${G}" ]; then
	echo "-usage=you can set them like: ${inform}"
	exit
fi

fresolution=R_svd_AG_${A%.dat}_${G%.dat}.dat

###----------------------------------------------------------
### inverse X on the basis of
###    forward equation:  A * X = B
###----------------------------------------------------------
rm -f ${fresolution}

#cat <<eof >a.m
octave <<eof &>/dev/null
clear
clc

A=load("${A}");
G=load("${G}");

[U S V] = svd(A, "econ");

[nrank m]=size(S);
R=zeros(nrank, nrank);
for i=1:nrank
	R(i,i)=1.0/S(i,i);
end

Ainv=V * R * U';

[nrow ncol] = size(Ainv);
[nrowG ncolG] = size(G);
G((nrowG+1):ncol, :)=0;

res=Ainv * G;

save -ascii ${fresolution} res

quit
eof

echo "resolution: ${fresolution}"

########################################################
### Draw resolution figure
########################################################
draw_resolution_matrix.sh -resf=${fresolution} -G=${G}

