#!/bin/bash
############################################################
###
### obtain a resolution matrix by SVD
###           forward equation: A * X = B
###           inverse equation: X = A^-1 * B
###          resolution matrix: R = A^-1 * A
###           
###   argus:
###           -A=?  A matrix file
###           -B=?  B vector file
###
###         by Meijian An, meijianan@live.com
###               Jan, 2012
###
############################################################

A=-
B=-
X=

inform="-A=A_matrix -B=B_vector (AX=B ==> X = A^-1 B)"
for (( i=1; i<=$#; i++ )); do
	argu="${!i}"
	case "${argu%%=*}" in
	"-A")      A="${argu#*=}";;
	"-B")      B="${argu#*=}";;
	"-X")      X="${argu#*=}";;
	esac
done

if [ ! -f "${A}" ] || [ ! -f "${B}" ]; then
	echo "-usage=you can set them like: ${inform}"
	exit
fi

fsolution=Xcalc_svd_${A%.dat}_${B%.dat}.dat
fresolution=R_svd_${A%.dat}.dat
###----------------------------------------------------------
### inverse X on the basis of
###    forward equation:  A * X = B
###----------------------------------------------------------
rm -f ${fsolution} ${fresolution}
#cat <<eof >a.m
octave <<eof &>/dev/null
clear
clc

A=load("${A}");
B=load("${B}");

%%%[U S V] = svd(A, 0);
[U S V] = svd(A, "econ");

[nrank m]=size(S);
R=zeros(nrank, nrank);
for i=1:nrank
	R(i,i)=1.0/S(i,i);
end

Ainv=V * R * U';
Xcal=Ainv * B;

save -ascii ${fsolution} Xcal

res=Ainv * A; %% = V * V';

save -ascii ${fresolution} res

%ixy=1
%r=res(ixy, :)';
%save -ascii R1_svd.dat r

quit

%pcolor(res)
%colorbar('East')
%ylabel("resolution")
eof

echo "  solution: ${fsolution}"
echo "resolution: ${fresolution}"

########################################################
### Draw resolution figure
########################################################
draw_resolution_matrix.sh -resf=${fresolution} -G=${A}

########################################################
###            Draw solution figure
########################################################
output=${fsolution%.dat}.ps
\rm -f ${output}

gmtset MEASURE_UNIT cm
gmtset CHAR_ENCODING Standard+
gmtset BASEMAP_TYPE FANCY
gmtset PLOT_DEGREE_FORMAT ddd:mm:ssF
gmtset FRAME_PEN 1p FRAME_WIDTH 0.1c
gmtset HEADER_FONT_SIZE	 14p	HEADER_OFFSET 0.6c  HEADER_FONT Times-Bold
gmtset TICK_LENGTH 0.15c TICK_PEN 1p
gmtset ANOT_FONT_SIZE 12p ANOT_OFFSET 0.1c 
gmtset LABEL_FONT_SIZE 12p LABEL_OFFSET 0c
gmtset COLOR_NAN 225/225/225 COLOR_FOREGROUND 200/200/200
#gmtset GRID_CROSS_SIZE 0
gmtset GRID_PEN  0.1p,gray

nx=`awk '(NR==1){print NF}' $A`
minmax=(`awk '{print $(NR)}' ${fresolution} | minmax -C | awk '{d=($2-$1)/5; if(d<0.1) d=0.1; if($2>=0.9) print -1.1, 1.1, 0.2; else print -($2+d), $2+d, ($2+d)/20}'`)
echo ${minmax[*]}
psbasemap -R0/$[nx+1]/${minmax[0]}/${minmax[1]} -JX15/7 -BnSWea10f1g5:"Parameter index":/a0.5f0.2g0.5:."Diag of resolution matrix":  -X3 -Y9 -K -P > ${output}
awk '{print NR, $(NR)}' ${fresolution} | psxy -R -J -W2/blue -Sc0.1c -O -K >>${output}

if [ -f "${X}" ]; then
	minmax=(`cat $X ${fsolution} | minmax -C | awk '{d=($2-$1)/5; print $1-d, $2+d, $2-$1}'`)
else
	minmax=(`cat  ${fsolution} | minmax -C | awk '{d=($2-$1)/5; print $1-d, $2+d, $2-$1}'`)
fi

psbasemap -R0/$[nx+1]/${minmax[0]}/${minmax[1]} -JX15/7 -BnSWea10f1g5:"Parameter index":/a0.5f0.1:."Solution":  -Y10 -K -O >> ${output}
if [ -f "${X}" ]; then
	awk '{print NR, $1}' $X | psxy -R -J  -Ss0.1c -W2,red -O -K >> ${output}
fi
awk '{print NR, $1}' ${fsolution} | psxy -R -J  -Sc0.1c -W2,blue -O >> ${output}

ps2pdf ${output}
xpdf ${output%.ps}.pdf&
rm -f ${output} .gmt*
