#!/bin/bash
########################################################
###
###      Calculate statistical resolution matrix
###           on the basis of SVD inversion.
###
###         by Meijian An, meijianan@live.com
###               Jan, 2012
###
########################################################

nsols=25   ##use 25 different solutions
if [ $# -ge 1 ]; then
	nsols=$1
fi

refer_value=1.25
bindir=../../   #/home/meijian/surface_wave/srcs_package
str=u_1  #o_1_5
G=G${str}.dat
A=A${str}.dat
X=X${str}.dat
mparam=`awk '(NR==1){print NF}' $G`
nob=`awk 'END{print NR}' $G`
nconst=`awk 'END{print NR}' $A | awk '{print $1 - '${nob}'}'`

### right values (B_C) for flatness constraints
#
# B= | D   |
#    | B_C |
#
rm -f B_C.dat
awk 'BEGIN{  for(i=1; i<='${nconst}'; i++)  print 0	}' > B_C.dat

####
rm -f synt_${str}_${nsols}_solutions.all synt_${str}.sol.* synt_${str}.ob.* synt_${str}.*.B

awk 'BEGIN{for(i=1; i<='${mparam}'; i++) print i}' > synt_${str}_${nsols}_solutions.all

for ((i=1; i<=${nsols}; i++)); do
	echo
	echo "make $i th synthetic and calc solution ..."
	
	## generate a random model as real/true solution
	awk 'BEGIN{
		srand('${i}')
		for(ixy=1; ixy<='${mparam}'; ixy++)
		{
			dsol=(rand() - 0.5) * 2 * ('${refer_value}' * 0.1)
			sol='${refer_value}' + dsol
				
			print sol
		}
	}' | tee synt_${str}.sol.${i}.real |\
		paste synt_${str}_${nsols}_solutions.all - >a; mv -f a synt_${str}_${nsols}_solutions.all
	
	## generate synthetic observation from the real solution
	awk 'BEGIN{
		mparam=0
		while(getline<"synt_'${str}'.sol.'${i}'.real")
		{
			if(NF>=1)
			{
				mparam ++;
				X[mparam] = $1
			}
		}
	}{
		ob=0
		for(icol=1; icol<=NF; icol++)
			ob += $(icol) * X[icol]
		print ob
	}' $G >synt_${str}.ob.${i}.D
	
	cat synt_${str}.ob.${i}.D B_C.dat > synt_${str}.${i}.B
	
	## ------------------------------------------------------------
	##  invert for solution by SVD, under flatness constraint
	## ------------------------------------------------------------
	octave <<eof &>/dev/null
clear
clc

A=load("${A}");
B=load("synt_${str}.${i}.B");

[U S V] = svd(A, "econ");

[nrank m]=size(S);
R=zeros(nrank, nrank);
for i=1:nrank
	R(i,i)=1.0/S(i,i);
end

Ainv=V * R * U';
Xcal=Ainv * B;

save -ascii synt_${str}.sol.${i}.calc Xcal
quit
eof
	
	paste synt_${str}_${nsols}_solutions.all synt_${str}.sol.${i}.calc >a; mv -f a synt_${str}_${nsols}_solutions.all
	echo "solution $i is done"
done

resf=R_svd_inverse_${str}_${nsols}.dat
res=R_svd_inverse_${str}_${nsols}

time ${bindir}get_statistical_resolution_length \
	-Xs=synt_${str}_${nsols}_solutions.all -Rs=resolution_${str}_${nsols}.dat -mx=${mparam} -nsols=${nsols} -dim=1 -log=resolution.log \
	-mRLen=30 -Rmatrix=${resf} -resolengths=5,20 -resolerr=2.5 #-normalize

###------------------------------------------------------
### Draw resolution figure
###------------------------------------------------------

draw_resolution_matrix.sh -resf=$resf

###------------------------------------------------------
###            Draw solution figure
###------------------------------------------------------
#resf=resolution_${nsols}.dat
#res=resolution_${nsols}
output=resolution_synt_${str}_${nsols}.ps
\rm -f ${output}

rm -f .gmt*
gmtset MEASURE_UNIT cm
gmtset CHAR_ENCODING Standard+
gmtset BASEMAP_TYPE FANCY
gmtset PLOT_DEGREE_FORMAT ddd:mm:ssF
gmtset FRAME_PEN 1p FRAME_WIDTH 0.1c
gmtset HEADER_FONT_SIZE	 14p	HEADER_OFFSET 0.6c  HEADER_FONT Times-Bold
gmtset TICK_LENGTH 0.15c TICK_PEN 1p
gmtset ANOT_FONT_SIZE 12p ANOT_OFFSET 0.1c 
gmtset LABEL_FONT_SIZE 12p LABEL_OFFSET 0c
gmtset COLOR_NAN 225/225/225 COLOR_FOREGROUND 200/200/200
#gmtset GRID_CROSS_SIZE 0
gmtset GRID_PEN  0.1p,gray

minmax=(`awk '{print $NR}' ${resf} | minmax -C | awk '{d=$2/10; if($2>=0.9) print -1.1, 1.1, 0.05; else printf "%.2lf %.2lf %.3lf\n", -($2+d), $2+d, ($2+d)/20.0}'`)

psbasemap -R0/$[mparam+1]/${minmax[0]}/${minmax[1]} -JX15/5 -BnSWea10f1g5:"Parameter index":/a0.1f0.05g0.1:."Diag of resolution matrix":  -X3 -Y5 -K -P > ${output}
awk '{print NR, $(NR)}' ${resf} | psxy -R -J -W2/blue -O -K >>${output}

minmax=(`awk '{print $2}' resolution_${str}_${nsols}.dat | minmax -C | awk '{d=($2-$1)/10; if(d==0) d=1; print $1-d, $2+d, d*2}'`)
psbasemap -R0/$[mparam+1]/${minmax[0]}/${minmax[1]} -JX15/5 -BnSWea10f1g5:"Parameter index":/a5f1g5:"Length"::."resolution length":  -Y9 -K -O >> ${output}
awk '{print $1, $2}' resolution_${str}_${nsols}.dat | psxy -R -J -W4,red -K -O >> ${output}

# 
# awk '{print NR, $1 *10}' X.dat |\
# 	psxy -R -J -W4,blue -K -O >> ${output}
# 
# awk '{print NR, $1 *10}' synt.sol.1.real |\
# 	psxy -R -J -W2,black -K -O >> ${output}
# 
# awk '{print NR, $1 *10}' synt.sol.1.calc |\
# 	psxy -R -J -W2,black,- -K -O >> ${output}

minmax=(`cat ${X} synt_${str}.sol.2.real synt_${str}.sol.2.calc | minmax -C | awk '{d=($2-$1)/5; print $1-d, $2+d, $2-$1}'`)
psbasemap -R0/$[mparam+1]/${minmax[0]}/${minmax[1]} -JX15/5 -BnSWea10f1g5:"Parameter index":/a0.5f0.1g0.5:."Solution examples":  -Y9 -K -O >> ${output}
if [ -f "${X}" ]; then
	awk '{print NR, $1}' $X | psxy -R -J -W2,red -O -K >> ${output}
fi
awk '{print FNR, $1}' synt_${str}.sol.2.real | psxy -R -J -m -W2,green -O -K>> ${output}
awk '{print FNR, $1}' synt_${str}.sol.2.calc | psxy -R -J -m -W2,blue -O >> ${output}

ps2pdf ${output}
xpdf ${output%.ps}.pdf&
rm -f ${output}

rm -f .gmt*

