#!/bin/bash
############################################################
###
###       make a synthetic overdetermined inverse problem
###            and calculate its resolution matrixes
###
###         by Meijian An, meijianan@live.com
###               Jan, 2012
###
############################################################

#bindir=/home/meijian/surface_wave/srcs_package
mparam=100
nob=5
weights=(1 10)
wstr=`echo ${weights[*]} | awk '{str=""; for(i=1; i<=NF; i++) str=str "_" $i; print str }'`

############################################################
### create a synthetic over-determined inverse problem
############################################################
octave <<eof &>/dev/null
clc
clear

%%-------------------------------------------------------------
%% observation (D) and forward/observation operator/matrix (G)
%%     G * X = D
%%
%% Regularized equation is:
%%     A * X = B
%% while matrix (A) and vector (B) are
%%     A = | G        |
%%         | Flatness |
%%
%%     B = | D |
%%         | 0 |
%%
%%-------------------------------------------------------------
mparam=${mparam};
nob=${nob} + mparam;

X = zeros(mparam,1);  %% grid
G = zeros(nob, mparam);

switch(mparam)
case 1000
	X(1:100) =1.0;
	X(101:300)=1.5;
	X(301:600)=1.4;
	X(601:800)=1.1;
	X(801:850)=1.0;
	X(851:1000)=1.1;
	
	G(1,  1:100)=1;
	G(2,  1:300)=1;
	G(3, 301:600)=1;
	G(4, 601:800)=1;
	G(5, 801:850)=1;
case 100
	X(1:10) =1.0;
	X(11:30)=1.5;
	X(31:60)=1.4;
	X(61:80)=1.1;
	X(81:85)=1.0;
   X(86:100)=1.1;
	
	G(1,  1:10)=1;
	G(2,  1:30)=1;
	G(3, 31:60)=1;
	G(4, 61:80)=1;
	G(5, 81:85)=1;
case 500
	X(1:50) =1.0;
	X(51:150)=1.5;
	X(151:300)=1.4;
	X(301:400)=1.1;
	X(401:425)=1.0;
	X(426:500)=1.1;
	
	G(1,   1:50 )=1;
	G(2,   1:150)=1;
	G(3, 151:300)=1;
	G(4, 301:400)=1;
	G(5, 401:425)=1;
endswitch

for i=1:mparam
	G(${nob} + i, i) =  ${weights[0]}; 
end

%%z = [ 10 30 60 80 90 100 ]';  %% observation position

A = zeros(nob + mparam -1, mparam);
A(1:nob, :) = G;

%% add flatness constraints
for i=2:mparam
	A(nob + i -1, i-1) =  ${weights[1]};
	A(nob + i -1, i  ) = -${weights[1]};
end

%%  get the synthetic observation vector
D = G * X;

B = zeros(nob+mparam-1, 1);
B(1:nob) = D;

save -ascii Xo${wstr}.dat X
save -ascii Do${wstr}.dat D
save -ascii Bo${wstr}.dat B
save -ascii Ao${wstr}.dat A
save -ascii Go${wstr}.dat G

%plot(1:mparam, X);
%hold on;
%plot(1:10,   1.10*ones(10), 'r');
%plot(1:30,   1.11*ones(30), 'k');
%plot(31:60,  1.12*ones(30), 'g');
%plot(61:80,  1.13*ones(20), 'r');
%plot(31:90,  1.14*ones(60), 'k');
%plot(81:100, 1.15*ones(20), 'g');
%axis([0 101 0.9 1.6]);
%xlabel( 'position' )
%ylabel( 'value' )
quit
eof

############################################################
### calculate three different resolution matrixes
############################################################
## get direct resoluiton matrix.
get_resolution_solution_svd.sh -A=Go${wstr}.dat -B=Do${wstr}.dat -X=Xo${wstr}.dat
#rename _svd. _svd_GD. *_svd.???
#mv -f R_svd_Normalized.pdf R_svd_GD_Normalized.pdf

## get regularized resoluiton matrix.
get_resolution_solution_svd.sh -A=Ao${wstr}.dat -B=Bo${wstr}.dat -X=Xo${wstr}.dat
#rename _svd. _svd_AB. *_svd.???
#mv -f R_svd_Normalized.pdf R_svd_AB_Normalized.pdf

## get hybrid resoluiton matrix.
get_resolution_AG_svd.sh -A=Ao${wstr}.dat -G=Go${wstr}.dat
