################################################################################
###
### This is a template to calculate statistical resolution lengths, as 
### suggested by Meijian An (2012, GJI). You need modify this file according
### to your real problem. Before the modification, a forward program and a 
### inversion program for your problem are needed.
###
###            By Meijian An, 01/09/2012
###
################################################################################
#!/bin/bash

nX=??  ##parameter number of your model
       ## ***You must set it according to your real problem***

nsols=300  ##sample number -- the number of synthetic models and solutions
nsols_first=1  ## if the code was run once, you can continue to run it
               ## by set nsols_first according to nsols in the last run.

model_ref=xyz_for_all_points  ## positions of all points in your model. 
                              ## ***You must set it according to your real problem***
value_refer=3.6   ## a reference value to make random synthetic models.
value_uncert=0.1  ## 10%. random uncertainty will be added on the reference value.

if [ ! -f models_solutions.all ]; then
	nsols_first=1
fi

################################################################################
###  Make a file (models_solutions.all) including all synthetic models 
###  and solutions. The file format is:
###  -----------------------------------------------------------------------------------------------------------------------------------------------
###  x y z model1(sol.1.real) solution1(sol.1.calc) model2(or sol_2.real) solution2(sol.2.calc) .... modeln(or sol_n.real) solutionn(sol.n.calc)
###  ...
###  -----------------------------------------------------------------------------------------------------------------------------------------------
################################################################################
if [ ${nsols_first} -le 1 ]; then
	nsols_first=1
	
	### copy X positions to models_solutions.all
    ### ***You must modify code below according to your real problem***
	awk '{print $1, $2, $3}' ${model_ref} > models_solutions.all
fi

for ((i=${nsols_first}; i<=${nsols}; i++)); do
	echo
	echo "make $i th synthetic real and calc solution ..."
	
	## generate a random solution
	awk 'BEGIN{
		srand('${i}')
		for(ix=1; ix<='${nX}'; ix++)
		{
			dsol=(rand() - 0.5)*2 * '${value_refer}' * '${value_uncert}'
			sol='${value_refer}' + dsol
		}
	}' >sol.${i}.real
	
	###-----------------------------------------------------------------------------------
	###            ***You must add codes according to your real problem***
	### forward and inversion calculation as your real inversion:
	###
	### (1) do forward calculation to make calculated observation (ob.${i}.real) 
	###     using your synthetic real model (sol.${i}.real) and your forward 
	###     codes.
	###
	### (2) invert for a solution (sol.${i}.calc) using ob.${i}.real and your
	###     a prior constraints as the inversion for your real problem.
	###           ***You must add codes according to your real problem***
	###-----------------------------------------------------------------------------------

	### add the model and solution to models_solutions.all
	paste models_solutions.all sol.${i}.real sol.${i}.calc >a; mv -f a models_solutions.all

	rm -f ob.${i}.real sol.${i}.real sol.${i}.calc
done

################################################################################
###
###   Grid search for resolution lengths for all points.
###
###   In order to improve the efficiency, you can set parameters:
###                 -resolengths=min,max 
###                 -resolerr=?
###   according to your real problem.
###
###           ***You should modify code below according to your real problem***
################################################################################
get_statistical_resolution_length \
	-Xs=models_solutions.all -Rs=resolution_length_${nsols}.dat -mx=${nX} -nsols=${nsols} -dim=3 \
	-log=hresolution_${nsols}.log ##-normalization=2 

echo "Done! output can be found in resolution_length_${nsols}.dat"
