/***************************************************************************
			 mystds.cpp: common functions
                           -------------------
    begin                : 2005/01/19
    copyright            : (C) 2005 by Meijian An
    email                : meijian@iag.usp.br
 ***************************************************************************/

//#include <iostream>
#ifdef WIN32
#include <direct.h>
#include  <io.h>
#else
#include <dirent.h>
#include <unistd.h>
#endif

#ifdef _OPENMP
#include <omp.h>
#endif

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>
#include <cstdio>
#include <cmath>
#include <cctype>    /*tolower()*/
#include <sys/types.h>
#include <sys/stat.h>
#include "mystds.h"

/* show Error or Warnings*/
FILE *fpLog=NULL;
bool openLog(const char *szFile, const char *mode)
{
	fpLog = fopen(szFile, mode);
	if(fpLog==NULL)	return false;
	return true;
}

void closeLog()
{
	if(fpLog==NULL)	fclose(fpLog);
	fpLog = NULL;
}

void ShowWarn(const char *szText, FILE *out)
{
#ifndef _WINDOWS
	fprintf(out, "%s", szText); //same as: printf("%s",szText);
#endif
	if(fpLog != NULL)
		fprintf(fpLog, "%s", szText);
}

string *strTrim(string &str)
{
	char const* delims = " \t\r\n";

	// trim leading whitespace
	string::size_type  notwhite = str.find_first_not_of(delims);
	str.erase(0, notwhite);

	// trim trailing whitespace
	notwhite = str.find_last_not_of(delims);
	str.erase(notwhite+1);
	return &str;
}

char *strTrim(char *cbIn)
{
	//trim blanks from right to left
	int i;
	for(i=(int)strlen(cbIn)-1; i>=0; i--)
	{
		if(iswgraph(cbIn[i])) 
			break; 
		cbIn[i]=0;
	}

	int iLen = (int)strlen(cbIn);
	char *pcbTmp = new char[iLen+1];
	char *pcb = pcbTmp;
	sprintf(pcbTmp,"%s",cbIn);

	//trim blanks from left to right
	for(i=0;i<iLen;i++)
	{
		if(iswgraph(cbIn[i])) 
			break; 
		pcb++;
	}
	if(pcb!=pcbTmp)
		sprintf(cbIn,"%s",pcb);

	delete []pcbTmp;
	return cbIn;
}


//open a FILE like C
FILE *openfile(const char * szDir, const char * szFileName, const char * mode, bool bWarn, bool bExit)
{
	char szName[MAX_PATH_LENGTH];
	strcpy(szName, szDir);
	if( !isDir(szName) ) return NULL;

	strcat(szName, szFileName);
	return openfile(szName, mode, bWarn, bExit);
}

FILE *openfile(const char * szFileName, const char * mode, bool bWarn, bool bExit)
{
	if(szFileName == NULL)
	{
		SHOW("you have not set the file name!\n");
		exit(0);
	}

	FILE * file = fopen(szFileName, mode);
	if(file==NULL)
	{
		if(bWarn) 
			SHOW1("%s can not be opened!\n",szFileName);
		if(bExit) exit(0);
	}
	return file;
}

//open a (i/o)fstream
bool openfile(ifstream *file, const char * szFileName, int mode, bool bWarn, char iosf)
{
	return openfile((fstream *)file, szFileName, mode, bWarn, iosf);
}

bool openfile(ifstream *file, const char * szDir, const char * szFileName, int mode, bool bWarn, char iosf)
{
	char szName[MAX_PATH_LENGTH];
	sprintf(szName, "%s%s", szDir, szFileName);
	return openfile((fstream *)file, szName, mode, bWarn, iosf);
}

bool openfile(ofstream *file, const char * szFileName, int mode, bool bWarn, char iosf)
{
	return openfile((fstream *)file, szFileName, mode, bWarn, iosf);
}

bool openfile(ofstream *file, const char * szDir, const char * szFileName, int mode, bool bWarn, char iosf)
{
	char szName[MAX_PATH_LENGTH];
	sprintf(szName, "%s%s", szDir, szFileName);
	return openfile((fstream *)file, szName, mode, bWarn, iosf);
}

bool openfile(fstream *file, const char * szDir, const char * szFileName, int mode, bool bWarn, char iosf)
{
	char szName[MAX_PATH_LENGTH];
	sprintf(szName, "%s%s", szDir, szFileName);
	return openfile(file, szName, mode, bWarn, iosf);
}

bool openfile(fstream *file, const char * szFileName, int mode, bool bWarn, char iosf)
{
	file->clear();

	if(szFileName == NULL)
	{
		SHOW("you have not set the file name!\n");
		exit(0);
	}

	if(iosf == 'i')
		((ifstream*)file)->open(szFileName, (ios_base::openmode)mode);
	else if(iosf == 'o')
		((ofstream*)file)->open(szFileName, (ios_base::openmode)mode);
	else
	{
		file->open(szFileName, (ios_base::openmode)mode);

		if(!(*file))
		{
			file->clear();
			file->open(szFileName, (ios_base::openmode)(mode | ios_base::trunc));
		}
	}

	if(!(*file))
	{
		if(bWarn) 
			SHOW1("%s can not be opened!\n", szFileName);
		return false;
	}

	file->clear();
	return true;
}

//check if it is a directory
bool isDir(string &strDir)
{
	if(strDir.length() < 1)	strDir = "."; //or getcwd to set currect directory

	//check and add CBDIR in the last.
	if( (*strDir.rbegin()) != CBDIR ) //directory name should be ended by CBDIR ('/' or '\')
	{
		;//strDir.insert( strDir.end(), CBDIR );
	}
	else 
		strDir.erase( strDir.end()-1 ); //remove the last CBDIR ('/' or '\') which is for WIN32

	//check if it is directory
	struct stat fst;
	bool ret = false; 
#ifndef WIN32
	//check home symbol in linux/unix
	if(strDir[0] == '~')
	{
		char *pHome = getenv("HOME");
		if(pHome != NULL)
			strDir = string(pHome) + strDir.substr(1);
	}

	if(lstat(strDir.c_str(), &fst) == 0)
	{
		if(S_ISDIR(fst.st_mode)) ret = true;
	}
#else
	if(stat(strDir.c_str(), &fst) == 0)
	{
		if(_S_IFDIR & fst.st_mode)  ret = true;
	}
#endif

	strDir.insert( strDir.end(), CBDIR );
	return  ret;
}

//check if it is a directory
bool isDir(char *szDir)
{
	if(szDir == NULL) return false;

	std::string strDir = szDir;
	bool ret = isDir(strDir);
	strcpy(szDir, strDir.c_str());

	return ret;
}

char* get_ith_column(const char *Src, char *szColumn, int ith, const char *szToken)
{
	//Notice: the first element has a position of 1, not 0.
	char szTmp[1024];
	if(strlen(Src) > 1023) sprintf(szTmp, "%1023s", Src);
	else strcpy(szTmp, Src);
	
	char *token = strtok( szTmp, szToken);
	for(int i=1; i< ith && token != NULL; i++)
	{
		token = strtok( NULL, szToken );
	}

	if(token != NULL)
	{
		strcpy(szColumn, token);
		return szColumn;
	}

	return NULL;
}

int get_Tokens(char *Src, const char *szToken)
{
	char szTmp[1024];
	if(strlen(Src) > 1023) sprintf(szTmp, "%1023s", Src);
	else strcpy(szTmp, Src);
	
	int ntoken = 0;
	for(char *token = strtok( szTmp, szToken); token != NULL && ntoken < 1024; )
	{
		ntoken ++;
		token = strtok( NULL, szToken );
	}

	return ntoken;
}

int getTokens(char *sz, char cb)
{
	int iret=0;
	while((sz=strchr(sz,cb)) != NULL)
	{
		iret++;
		sz++;
	}
	return iret;
}

double getDistance(double *point1, double *point2, int dimension)
{
	double dist = 0.0;
	for(int idim = 0; idim < dimension; idim++)
		dist += pow(point1[idim] - point2[idim], 2);
	return pow(dist, 0.5);	
}


bool get_switch(const char* sz_param_switch, bool vDefault, int argc, char** argv)
{
	bool value = vDefault;
	for(int i=1; i<argc; i++)
	{
		if(strcmp( sz_param_switch, argv[i]) == 0)
		{
			value = true;
			break;
		}
	}
	
	return value;
}

#ifdef _OPENMP
int set_openmp_parallel_process_num(int nproc, bool force)
{
	if(nproc <= 0 && force == false)
	{
		int ncore = omp_get_num_procs(); 
		nproc = (ncore >2) ? (ncore - 1):1;
	}

	if(force==true || nproc>1)
	{
		omp_set_num_threads( nproc );
		fprintf(stderr, "Notice: This program is running in parallel mode and using %d threads!!\n", nproc);
	}

	return nproc;
}
#endif


